#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define VENDOR_RQ_SET_LED    0
#define VENDOR_RQ_GET_SWITCH 1
#define VENDOR_RQ_SET_LED1   2
/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
/* ------------------------------------------------------------------------- */

uint8_t in_c(uint8_t pin)
{
    // read logic level of port-C's pin
    DDRC &= ~(1<<pin);
    return ((PINC & (1<<pin))>>pin);    
} 

void out_c(uint8_t pin, uint8_t val)
{
    // write logic level to port-C's pin
    DDRC |= 1<<pin;
    if(val)
        PORTC |= 1<<pin;
    else
        PORTC &= ~(1<<pin);
}

uint8_t in_b(uint8_t pin)
{
    // read logic level of port-D's pin
    DDRB &= ~(1<<pin);
    return ((PINB & (1<<pin))>>pin);
}

void out_b(uint8_t pin, uint8_t val)
{
    // write logic level to port-C's pin
    DDRB |= 1<<pin;
    if(val)
        PORTB |= 1<<pin;
    else
        PORTB &= ~(1<<pin);
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
    usbRequest_t *rq = (void *)data;
    static uchar dataBuffer[6];  /* must stay valid when usbFunctionSetup returns */

    if (rq->bRequest == VENDOR_RQ_SET_LED)
    {
        uint8_t led_no  = rq->wValue.bytes[0];
        uint8_t led_val = rq->wValue.bytes[1];
        out_c(led_no, led_val);
        return 0;
    }
    else if (rq->bRequest == VENDOR_RQ_SET_LED1)
    {
        uint8_t led_no  = rq->wValue.bytes[0];
        uint8_t led_val = rq->wValue.bytes[1];
        out_b(led_no, led_val);
        return 0;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH)
    {
        dataBuffer[0] = !in_c(0);
        dataBuffer[1] = !in_c(1);
        dataBuffer[2] = !in_c(2);
        dataBuffer[3] = !in_b(0);
        dataBuffer[4] = !in_b(1);
        dataBuffer[5] = !in_b(2);
        usbMsgPtr = dataBuffer;
        return 6;
    }
    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{ 
    DDRC  = 0b00011000;  
    PORTC = 0b00000000;  
    PORTC |= (1 << 0);
    PORTC |= (1 << 1);
    PORTC |= (1 << 2);
    DDRB  = 0b00011000; 
    PORTB = 0b00000000;
    PORTB |= (1 << 0);
    PORTB |= (1 << 1);
    PORTB |= (1 << 2);   
    DDRD  = 0b00001000;
    PORTD = 0b00000000;
    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(300);
    usbDeviceConnect();
    sei();
    for(;;){                /* main event loop */
        usbPoll();
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
